-- GetLog PLugin

-- Constants
local GL_SID = "urn:srs-com:serviceId:GetLog"

-- PLUGIN_NAME serves several purposes - do not change
local PLUGIN_NAME = "GetLogV1.3"
-- debug message prefix
local PRE = PLUGIN_NAME .. " device: "
-- file names and paths
local BASEPATH = "/etc/cmh-ludl/" -- default vera directory
local LOGFILE = "/var/log/cmh/LuaUPnP.log"
local LOGFILECOPY = "/var/log/cmh/00-GetLog.log"
local LOGFILECOMPRESSED = BASEPATH .. "00-GetLog.log.lzo"
local GCAL3LOGS = BASEPATH .. "*GCal3.log.lzo"
local TEMPFILE = ""
-----------------------------
-- Utility local functions
-----------------------------

function DEBUG(s)
  s = PRE .." - " .. s
  luup.log(s)
end

local function osExecute(command)
  local result = os.execute(command)
  local msg = 'Command ' .. command .. ' returned ' .. tostring(result)
  DEBUG(msg)
  return result
end

function trimString( s )
  return string.match( s,"^()%s*$") and "" or string.match(s,"^%s*(.*%S)" )
end

-- Main Copy Log function
function clearLogs()
  luup.variable_set(GL_SID, "gl_Error","Clearing Log Files", lul_device)
  DEBUG("*****************")
  DEBUG("Clearing Log Files")
  DEBUG("*****************")
  -- get rid of all GCal3 log files
  local command = "/bin/rm -f " .. GCAL3LOGS
  local _ = osExecute(command)
  command = "/bin/rm -f " .. LOGFILECOMPRESSED
  local _ = osExecute(command)
end
function copyLog()
  luup.variable_set(GL_SID, "gl_Error","Creating ....", lul_device)
  DEBUG("*****************")
  DEBUG("GetLog file created")
  DEBUG("*****************")

  local errormsg = ""
  -- get rid of old compressed file
  local command = "/bin/rm -f " .. LOGFILECOMPRESSED
  local result = osExecute(command)
  -- flush the write buffer
  command = "sync"
  result = osExecute(command)
  -- get the log entries requested by the user
  local pattern = luup.variable_get(GL_SID, "gl_Grep", lul_device)
  pattern = pattern or ""
  if pattern == "" then -- no grep pattern so just use log file
    TEMPFILE = LOGFILE
  else
    TEMPFILE = LOGFILECOPY
    -- command = "grep " .. pattern .. " " .. LOGFILE .. " > " .. TEMPFILE
    command = "cat " .. LOGFILE .. " | grep " .. pattern .. " > " .. TEMPFILE
    result = osExecute(command)
    if result ~= 0 then
      luup.variable_set(GL_SID, "gl_Error","Grep Error Code: " .. tostring(result) , lul_device)
      return false
    end
  end
  -- compress the log and place it in the accessible directory
  command = "pluto-lzo c " .. TEMPFILE .. " " .. LOGFILECOMPRESSED
  result = osExecute(command)
  if (result ~= 0) then
    errormsg ="Failed to create: " .. LOGFILECOMPRESSED .. " : " .. tostring(result)
    DEBUG(errormsg)
  end
  -- remove the uncompressed version if one was created
  if (LOGFILE ~= LOGFILECOPY) then
    command = "/bin/rm -f " .. LOGFILECOPY
    result = osExecute(command)
  end 
  luup.variable_set(GL_SID, "gl_Error","Log File Created", lul_device)
  return true
end

-- Initialization Function
function GetLogStartup()
  luup.set_failure(false) -- just to clear any prior issues
  PRE = PRE .. tostring(luup.device) -- debug message prefix
  DEBUG("local function: GetLogStartup")
  -- Get rid of all the old log files on reboot to keep things clean
  local command = "/bin/rm -f " .. LOGFILECOPY .. ".lzo"
  local _ = osExecute(command)
  -- Initialize Variables
  luup.variable_set(GL_SID, "gl_Error","Starting ....", lul_device)

  -- set the correct UI for Vera Version
  local UI7Check = "false"
  local s1 = ""
  s1 = luup.variable_get(GL_SID, "gl_UI7Check", lul_device)
  if s1 == "true" then UI7Check = "true" end

  if ( luup.version_branch == 1 and luup.version_major == 7 and UI7Check == "false") then
    luup.attr_set("device_json", "D_GetLog_UI7.json", lul_device)
    -- save any changes to GCV that happened as part of setup
    luup.variable_set(GL_SID, "gl_UI7Check","true", lul_device)
    local errormsg = "Configured UI7 file"
    DEBUG(errormsg)
    luup.variable_set(GL_SID, "gl_Error",errormsg, lul_device)
    luup.reload()
  end
  luup.variable_set(GL_SID, "gl_Error","", lul_device)
  return true
end
